% *************************************************************
% author:   Gene Cheung                                      **
% date:     10/28/2012                                       **
% modified: 07/24/2014                                       **
% purpose:  arithmetic edge decoding                         **
% *************************************************************

function [dcode] = aed (d, symCount)
% *** Arithmetic Edge Decoder ***
% input: AEC codeword (d) and the number of symbols (symCount),
% output: direction code (1-4 means North, East, South, West).

% step 0: initialize var
dcode = [];
fitLength = 3;    % length of input vector for bestFitLine

b = 0; % interval base
l = 1; % interval length
P = min(32,length(d));  % 32-bit precision
t = P;  % P-bit precision
v = 0; % code value
for n = 1:P
    v = v + d(n)*2^(-n);
end

for ind = 1:symCount
    
    % step 1: find best estimate of prob of 3 poss. directions ********
    % ******** (must be same as encoder in aec.m) ********
    if ind == 1,
        prob = ones(1,4) / 4.0;    % no end-code for first chain
    else
        if ind > fitLength,
            prob = bestFitLine(dcode(ind-fitLength:ind-1));
            % prob = bestFitLine2(dcode(1:ind-1));
        else
            prob = ones(1,3) / 3.0;
        end
        % add end-code prob
        endProb = 0;   % can be any function of ind
        prob = [prob*(1.0-endProb)  endProb];
    end
    % ******** (must be same as encoder in aec.m) ********
    
    
    % step 2: arithmetic decoding:  given prob, d, compute ind ********
    i = 1;
    while b + sum(prob(1:i))*l < v,  % search for the right symbol
        i = i + 1;
        if i > 4
            error = 1;
        end
    end
    
    if isempty(dcode) == 1,                % first ccode is abs ******
        dcode = [i];
        
    else
    
        if dcode(length(dcode)) == 1,      % North ******
            if i == 1,       % left
                dcode = [dcode 4];
            elseif i == 2,   % straight
                dcode = [dcode 1];
            else             % right
                dcode = [dcode 2];
            end
        elseif dcode(length(dcode)) == 2,  % East ******
            if i == 1,       % left
                dcode = [dcode 1];
            elseif i == 2,   % straight
                dcode = [dcode 2];
            else             % right
                dcode = [dcode 3];
            end
        elseif dcode(length(dcode)) == 3,  % South ******
            if i == 1,       % left
                dcode = [dcode 2];
            elseif i == 2,   % straight
                dcode = [dcode 3];
            else             % right
                dcode = [dcode 4];
            end
        else                               % West ******
            if i == 1,       % left
                dcode = [dcode 3];
            elseif i == 2,   % straight
                dcode = [dcode 4];
            else             % right
                dcode = [dcode 1];
            end
        end

    end
    
    % update interval
    b = b + sum(prob(1:i-1))*l;
    l = prob(i)*l;
    
    % check overflow
    if b >= 1 
        b = b - 1;  % shift interval base
        v = v - 1;  % shift code value
    end
    
    % renormalize interval
    while l <= 0.5
        if b >= 0.5   % remove most significant bit 
            b = 2*(b-0.5); % shift and scale interval base
            v = 2*(v-0.5); % shift and scale code value
        else
            b = 2*b; % scale interval base
            v = 2*v; % scale code value
        end

        if length(d) > P
            t = t+1; % increment bit counter
            if t <= length(d)
                v = v + (2^(-P))*d(t); % set least significant bit of code value
            end
        end
        
        l = 2*l; % scale interval length
    end
    
end
