% *************************************************************
% author:   Gene Cheung                                      **
% date:     07/23/2014                                       **
% modified: 07/29/2014                                       **
% purpose:  convert gradient maps to contours                **
% *************************************************************

function [startPt, dcode] = edgeMap2contour (hGrad, vGrad)

[maxHRow, maxHCol] = size(hGrad);
[maxVRow, maxVCol] = size(vGrad);
startPt = [];
dcode = [];

% there is edge not accounted for ********
while max(hGrad(:)) + max(vGrad(:)) > 0,  
    
    [dHeight, dWidth] = size(dcode);
    
    if dHeight == 8,
        ii = 1;
    end
    
    % step 0: initialize either vertical / horizontal edge
    if max(hGrad(:)) > 0,
        lastDirect = 3;        
        [rowV, colV] = find(hGrad > 0);
        [val, ind] = min(rowV);       % to go south, pick top row ***
        row = rowV(ind);
        col = colV(ind);
        startPt(dHeight+1).hGrad = 1;
        dcode(dHeight+1,1) = 3;       % south
        hGrad(row, col) = 0;
    else
        lastDirect = 2;
        [rowV, colV] = find(vGrad > 0);
        [val, ind] = min(colV);       % to go east, pick most left col ***
        row = rowV(ind);
        col = colV(ind);
        startPt(dHeight+1).hGrad = 0;
        dcode(dHeight+1,1) = 2;   % east
        vGrad(row, col) = 0;
    end
    
    startPt(dHeight+1).row = row;
    startPt(dHeight+1).col = col;
    
    dCount = 1;
    done = 0;
    
    % step 1: identify the rest of contour 
    while done == 0,
        
        if row == 406 && col == 528,
            ii = 1;
        end
        
        if lastDirect == 1,  % last direction is north ********
            if row > 1 && hGrad(row-1, col) > 0,
                hGrad(row-1, col) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 1;  % north
                row = row-1;
            elseif row > 1 && col < maxVCol && vGrad(row-1, col+1) > 0,
                vGrad(row-1, col+1) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 2;  % east
                lastDirect = 2;
                row = row-1;
                col = col+1;
            elseif row > 1 && vGrad(row-1, col) > 0,
                vGrad(row-1, col) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 4;  % west
                lastDirect = 4;
                row = row-1;
            else
                done = 1;
            end
        
        elseif lastDirect == 2,  % last direction is east ********
            if col <= maxHCol && hGrad(row, col) > 0,
                hGrad(row, col) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 1;  % north
                lastDirect = 1;
            elseif col < maxVCol && vGrad(row, col+1) > 0,
                vGrad(row, col+1) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 2;  % east
                col = col+1;
            elseif row < maxHRow && col <= maxHCol && hGrad(row+1, col) > 0,
                hGrad(row+1, col) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 3;  % south
                lastDirect = 3;
                row = row+1;            
            else
                done = 1;
            end
            
        elseif lastDirect == 3,  % last direction is south ********
            
            if row < maxHRow && hGrad(row+1, col) > 0,
                hGrad(row+1, col) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 3;  % south
                row = row+1;
            elseif row <= maxVRow && col < maxVCol && vGrad(row, col+1) > 0,
                vGrad(row, col+1) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 2;  % east
                lastDirect = 2;
                col = col+1;
            elseif row <= maxVRow && vGrad(row, col) > 0,
                vGrad(row, col) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 4;  % west
                lastDirect = 4;
            else
                done = 1;
            end
            
        elseif lastDirect == 4,  % last direction is west ********s
            if col > 1 && hGrad(row, col-1) > 0,
                hGrad(row, col-1) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 1;  % north
                lastDirect = 1;
                col = col-1;
            elseif col > 1 && vGrad(row, col-1) > 0,
                vGrad(row, col-1) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 4;  % west
                col = col-1;
            elseif row < maxHRow && col > 1 && hGrad(row+1, col-1) > 0,
                hGrad(row+1, col-1) = 0;
                dCount = dCount + 1;
                dcode(dHeight+1, dCount) = 3;  % south
                lastDirect = 3;
                row = row+1;
                col = col-1;
            else
                done = 1;
            end 
            
        end
    end
        
    dCount = dCount + 1;
    dcode(dHeight+1, dCount) = 0;   % end symbol
end