% *************************************************************
% author:   Gene Cheung                                      **
% date:     07/22/2014                                       **
% modified: 01/11/2015                                       **
% purpose:  find best approximate cost                       **
% *************************************************************

bSize = 16;

fRoot0 = 'Dancer/Dancer_d1_720_00';
fRoot = 'Dancer/Dancer_d1_720_0';
fStat = 'Dancer/Dancer_d1_720_edgeCost';
fEx = '.pgm';
%fRoot0 = 'Ballet/depth-cam0-f00';
%fRoot = 'Ballet/depth-cam0-f0';
%fStat = 'Ballet/depth-cam0-edgeCost';
%fEx = '.png';

fStatID = fopen(fStat, 'a');

for frame = 1:2,
    
    if frame < 10,
        fName = strcat(fRoot0, int2str(frame));
    else
        fName = strcat(fRoot, int2str(frame));
    end
    f1 = strcat(fName, fEx);
    
    im1 = imread(f1);
    im1 = double(im1(:,:,1));   % take the first component if png format
    [imRow, imCol] = size(im1);


    % step 1: compute horizontal, vertical gradients ********
    hGrad = zeros(imRow,imCol-1);  % horizontal gradient
    vGrad = zeros(imRow-1,imCol);  % vertical gradients
    for row = 1:imRow,
        hGrad(row,:) = abs(im1(row, 1:imCol-1) - im1(row, 2:imCol));
    end
    for col = 1:imCol,
        vGrad(:,col) = abs(im1(1:imRow-1,col) - im1(2:imRow,col));
    end
    gMean = mean([hGrad(:)' vGrad(:)']);
    gVar = var([hGrad(:)' vGrad(:)']);
    thresh = gMean + 1.0*sqrt(gVar);            % mean + 1.0 std
    hGrad = hGrad > thresh;
    vGrad = vGrad > thresh;
    

    % step 2: read out contour for encoding ********
    if frame > 1,
        prevStartPt0 = startPt0;  % preserve previous detected contours
        prevDcode0 = dcode0;     
    end
    [startPt0, dcode0] = edgeMap2contour(hGrad, vGrad);
    [dRow, dCol] = size(dcode0);
    clear startPt;
    dcode = [];
    count = 0;
    for i = 1:dRow,
        ind = find(dcode0(i,:) == 0);
        rep = 0;
        if frame > 1 && ind(1)-1 >= bSize,
            rep = repeatContour(prevStartPt0, prevDcode0, startPt0(i), dcode0(i,1:ind-1));
        end
        
        if rep == 0 && ind(1)-1 >= bSize,   % remove contour with length < bSize
            count = count+1;
            startPt(count).hGrad = startPt0(i).hGrad;
            startPt(count).row = startPt0(i).row;
            startPt(count).col = startPt0(i).col;
            dcode(count, :) = dcode0(i,:);
        end
    end
    
    figure(1);
    [hGrad2, vGrad2] = contour2edgeMap(startPt, dcode, imRow, imCol);
    displayEdgeMap(hGrad2, vGrad2, im1);

    % step 3: compute approximated contour ********
    % not done yet.


    % step 4: compute actual encoding cost ********
    symCount = 0;   % number of input symbols 
    bCount = 0;     % number of output coding bits
    [dRow, dCol] = size(dcode);
    for i=1:dRow,

        % step 4.1: AEC for each contour ***
        [val, ind] = min(dcode(i,:));
        dcode1 = dcode(i,1:ind-1);         % read the i-th contour
        [buffer, bCount1] = aec (dcode1);  % encoding 
        [base2,symNum] = deci2bin(ind-1);  % represent the length of the contour in binary
        bCount1 = bCount1 + symNum;        % add the bits to record the length of the contour
    
        % step 4.2: accumulate bits and symbols ***
        bCount = bCount + bCount1;  
        symCount = symCount + ind-1; 
        fprintf(fStatID, '%d\t%d\t%d\t%d\t%d\t%d\n', frame, horizon, startPt(i).hGrad, startPt(i).row, startPt(i).col, bCount1);
    end
    perf = bCount / symCount      % compute number of bits per symbol
    
    % step 5: output computed contour as gradient maps ********
    [hGrad2, vGrad2] = contour2edgeMap(startPt, dcode, imRow, imCol);
    printEdgeMap(hGrad2, vGrad2, fName, horizon);
    
    % step 6: visual check ********
    figure(2);
    displayEdgeMap(hGrad2, vGrad2, im1);

end

fclose(fStatID);
